%
clear all;
close all;
clc;

X = randn(500, 3);
ytrue = 0.4*X(:,1) - 2*X(:,2).^2 + 0.05*X(:,3).^3;
y = ytrue + 0.1*randn(500, 1);


%% select inputs and ouptut dimensions
[n, M] = size(X);

figure;
plot(X(:,1), y,  '.',  'linewidth',3); 
ylabel('output');
xlabel('feature 1');

figure;
plot(X(:,2), y, '.r', 'linewidth',3); 
ylabel('output');
xlabel('feature 2');

figure;
plot(X(:,3), y, '.g', 'linewidth',3); 
ylabel('output');
xlabel('feature 3');

%% Settings
%n = 10000; %max num samples
eta = 0.1;
lambda = 1e-3;

%%Model 
eFn = @(t, y)( 1/size(t,1)*(t-y)'*(t-y) );
w_func = @(A, lambda, trainy)(A'*A + lambda)^(-1) * A' * trainy;
w_opt = w_func(X(1:n,:), lambda, ytrue(1:n))';

%% init the model with the preview data
step = 1;
w(1,:) = 0.2*randn(1,3);
error(1) = eFn(ytrue, X*w(1,:)');
numW = 1;
%%sequential learning
for tauId=1:n-1
    numW = numW + 1; 
    
    xops = X(tauId,:);
    A = xops;
    yops = y(tauId); 
    wOld = w(numW-1,:)';
    
    deltaW = A'*(A*wOld - yops);
    
    %bound the update 
    %deltaW = max(min(deltaW, [1, 1, 1]), [-1, -1, -1]);
    
    w(numW,:) = w(numW-1,:) - eta*deltaW';
    error(numW) = eFn(ytrue, X*w(numW,:)');
    
end

%% seq. learning plot results
cLines = lines(3);

figure; hold all;
plot(w, 'linewidth', 3);
l1=plot([1, numW-1], [w_opt(1), w_opt(1)], 'linewidth', 2);
set(l1, 'Color', cLines(1,:));
l2=plot([1, numW-1], [w_opt(2), w_opt(2)], 'linewidth', 2);
set(l2, 'Color', cLines(2,:));
l3=plot([1, numW-1], [w_opt(3), w_opt(3)], 'linewidth', 2);
set(l3, 'Color', cLines(3,:));
%legend('w estimated', 'optimal values');
xlabel('sequential training sample');
ylabel('w vector');
set(gca, 'fontsize', 20);

figure; 
plot(error, 'linewidth', 3);
legend('MSE');
xlabel('sequential training sample');
ylabel('true error');
set(gca, 'fontsize', 20);
